"use strict";

// utilGenCam_rtc
//--------------
function utilGenCam_rtc(v_sourceid, previewVid, maxDim, afterVideoResize) {
	this._video = previewVid;
	this._v_sourceid = v_sourceid;

	this._maxDim = maxDim;
	this._width = 10;    // We will scale the photo (i.e. video) width to this
	this._height = 10;     // This will be computed based on the input stream

	this._afterVideoResize = afterVideoResize;

	this._isPortrait = null;
	this._streaming = false;

	navigator.getUserMedia = ( navigator.getUserMedia ||
					   navigator.webkitGetUserMedia ||
					   navigator.mozGetUserMedia ||
					   navigator.msGetUserMedia);

}


utilGenCam_rtc.prototype._onWindowResize = function() {
	var self = this;
	
	// intermittent wrong XY-dims problem (on video element - i.e. video input sometimes takes N-ms to change orientations):
	// - Updating XY-dims here is right ~90% of the time.
	// - Updating in orientationchange is wrong ~90% of the time.
	// - Updating in both places is also wrong sometimes.
	// - Trying setTimeout() here (Update: it works.)
	if (($(window).height() > $(window).width() && self._isPortrait !== true)
		|| ($(window).height() < $(window).width() && self._isPortrait !== false)) {
		setTimeout( function() {
			//g_genUI.TransientMsg("orient1", "Orientation1");
			self._isPortrait = $(window).height() > $(window).width();
			//alert(self._isPortrait);
			self._setupXYDims();
			if (self._afterVideoResize) {
				//g_genUI.TransientMsg("orient1", "Orientation2");

				self._afterVideoResize();
			}
		}, 100);
		
		//g_genUI.TransientMsg("orient3", "Orientation - by dim");
	}
}

utilGenCam_rtc.prototype._setupXYDims = function() {
	var self = this;
	var vidHtoW = self._video.videoHeight / self._video.videoWidth;
	if (self._video.videoHeight > self._video.videoWidth) { // portrait
		self._height = self._maxDim;
		self._width = self._maxDim / vidHtoW;
	}
	else { // landscape
		self._width = self._maxDim;
		self._height = self._maxDim * vidHtoW;
	}
	//self._height = self._video.videoHeight / (self._video.videoWidth/self._width);

	// Firefox currently has a bug where the height can't be read from
	// the video, so we will make assumptions if this happens.
	if (isNaN(self._height)) {
		self._height = self._width / (4/3);
	}

	var height = self._height;
	var width = self._width;
	self._video.setAttribute('width', width);
	self._video.setAttribute('height', height);

	/*g_genUI.TransientMsg("o-change2", "Orient: " + $(window).width() + "x" + $(window).height() +
		"<br/>vid: " + self._video.videoWidth + "x" + self._video.videoHeight +
		"<br/>vid-css: " + $(self._video).width() + "x" + $(self._video).height());*/
}


// Partly from: https://developer.mozilla.org/en-US/docs/Web/API/WebRTC_API/Taking_still_photos
// 				https://github.com/mdn/samples-server/blob/master/s/webrtc-capturestill/capture.js
utilGenCam_rtc.prototype.startUp = function(callback) {
	var self = this;
	var video_val = true;
	video_val = {optional: [
			{ facingMode: "environment" } // Default to rear cam - not supported by Android :(
		]};
	if (this._v_sourceid) { // In practice, this is always true (as of this writing)
		//alert("source:" + this._v_sourceid);
		//g_genUI.TransientMsg("v_source", "v_source: " + this._v_sourceid);
		video_val = {optional: [
			{ sourceId: this._v_sourceid }, 
		]};
	}
    navigator.getUserMedia(
		{
			video: video_val,
			audio: false
		},
		function(stream) {
			self._mediaStream = stream;
			if (navigator.mozGetUserMedia) {
				self._video.mozSrcObject = stream;
			} else {
				var vendorURL = window.URL || window.webkitURL;
				self._video.src = vendorURL.createObjectURL(stream);
			}
			//g_genUI.TransientMsg("initCam_starting", "Starting camera");
			//self._video.play();
		},
		function(err) {
			console.log("getUserMedia error: " + err);
			g_genUI.TransientMsg("initCam_err1", "getUserMedia error: " + err);
		}
	);

    self._video.addEventListener('canplay', function(ev){
		if (!self._streaming) {
			self._setupXYDims();
			self._streaming = true;
			//self._video.play();
			//g_genUI.TransientMsg("initCam_canplay", "canplay3");
			//self.startPreview();
			if (callback)
				callback(); // startPreview() from outside.
		}
	}, false);

	// Initialize window resize listener for vid resize "events".
	setTimeout(function() { self._onWindowResize(); }, 50);
	$(window).on("resize", function() {
		self._onWindowResize();
	});

}



utilGenCam_rtc.prototype.startPreview = function() {
	var self = this;

	/*var vidTracks = self._mediaStream.getVideoTracks(); // Unsuccessful attempt to save battery when off, 20160611
	if (vidTracks[0])
		vidTracks[0].enabled = true;*/

	//this._streaming = true;
	this._video.play();

	//g_genUI.TransientMsg("initCam_start", "Camera start - " + this._video.width + "," + this._video.height);
}
utilGenCam_rtc.prototype.stopPreview = function() {
	var self = this;
	//this._streaming = false;
	this._video.pause();

	/*var vidTracks = self._mediaStream.getVideoTracks(); // unsuccessful attempt to save battery when off, 20160611
	if (vidTracks[0])
		vidTracks[0].enabled = false;*/
}

utilGenCam_rtc.prototype.frameCount = function() {
	if (this._video.webkitDecodedFrameCount === undefined)
		alert("Error: webkitDecodedFrameCount not found");
	return this._video.webkitDecodedFrameCount;
	//return this._video.getVideoPlaybackQuality().totalVideoFrames; // moz-
}



utilGenCam_rtc.prototype.drawToCanvas = function(cnvs, ctx) {
	if (!this._width || !this._height) {
		alert("error - rtcCam not initialized.");
		return;
	}
	if (cnvs.width != this._width || cnvs.height != this._height) {
		cnvs.width = this._width;
		cnvs.height = this._height;
	}
	ctx.drawImage(this._video, 0, 0, this._width, this._height);
}
