"use strict";

// utilGenCam_svr
// BL-conventional way to access cordova-camera-server plugin (test, for iOS).
//--------------
function utilGenCam_svr($liveCnvs, maxDim, afterVideoResize) {

	this._$liveCnvs = $liveCnvs;
	if ($liveCnvs.length) {
		this._liveCnvs = $liveCnvs.get(0);
		this._liveCtx = this._liveCnvs.getContext('2d');
	}
	else 
		this._$liveCnvs = document.createElement('canvas');

	
	this._afterVideoResize = afterVideoResize;
	//this._onFrameAdvance = onFrameAdvance;
	
	
	// Test
	//this._stgCtx.fillStyle="#FF0000";
	//this._stgCtx.fillRect(0,0,this._stgCnvs.width,this._stgCnvs.height); 

	this._frameCount = 0;
	this._maxDim = maxDim;
	this._prevOrientation = "asdf";
	this._lastImgOrientation = "";
	this.echoFrameCount = false;
	
	this._cameraSvrUrl = null;
}


utilGenCam_svr.prototype._camServerStart = function(callback) {
	var self = this;

	// before start, check whether its up or not
	// (copied from reference app com.moonware.smart)
	// (Possibly avoids "CameraServer Start failed: server is already up" common error on iOS startup.)
	cordova.plugins.CameraServer.getURL(function(url){
		if(url.length > 0) {
			g_genUI.TransientMsg("cs-serverstart", "Camera restart - already running: " + url);
		} else {
			/* wwwroot is the root dir of web server, it can be absolute or relative path
			 * if a relative path is given, it will be relative to cordova assets/www/ in APK.
			 * "", by default, it will point to cordova assets/www/, it's good to use 'htdocs' for 'www/htdocs'
			 * if a absolute path is given, it will access file system.
			 * "/", set the root dir as the www root, it maybe a security issue, but very powerful to browse all dir
			 */
			cordova.plugins.CameraServer.startServer({
				'www_root' : '/',
				'port' : 8080,
				'localhost_only' : false,
				'json_info': []
			}, function( svrUrl ){
				//g_genUI.TransientMsg("cs-startserver-s", "cs-startserver-success " + svrUrl);
				self._cameraSvrUrl = svrUrl;
				if (callback)
					callback();
			}, function( error ){
				// Just TransientMsg(), because this error frequently goes away after a bit, and alert() blocks it from fixing itself.
				//alert('CameraServer Start failed: ' + error);
				g_genUI.TransientMsg("startFail1", 'CameraServer Start failed: ' + error);
			});
		}
	});
};

utilGenCam_svr.prototype.camServerRestart = function(callback) {
	var self = this;
	cordova.plugins.CameraServer.stopServer(
		function() {
			//g_genUI.TransientMsg("cs-stopserver", "cs-stopserver success");
			self._camServerStart(callback);
		}, function( error ){
			alert('CameraServer Stop failed: ' + error);
		}
	);
};

utilGenCam_svr.prototype.cameraRestart = function(callback) {
	cordova.plugins.CameraServer.stopCamera(function () {
		cordova.plugins.CameraServer.startCamera(function(){
			console.log('Capture Started');
			if (callback)
				callback();
		},function( error ){
			alert('CameraServer StartCapture failed: ' + error);
		});
	}, function( error ){
		alert('CameraServer StopCapture failed: ' + error);
	});
}

utilGenCam_svr.prototype.startUp = function(callback) {
	var self = this;
	
	// Start camera server
	self.camServerRestart( function() {
		// Start camera itself
		self.cameraRestart( function() {
			// Will always fail initially(?), but will recover on image1.onerror in _getImage().
			//self.startPreview();
			if (callback)
				callback(); // startPreview() from outside.
		});
	});

	// Camera-server camera does not auto-handle device orientation changes.  However, correctly detects orientation on cameraStart.
	// So, detect orientation change, and force camera restart.
	//self._prevOrientation = window.orientation;
	setInterval(function() {
		// At least notify if orientation is messed up.
		// Result: imgOrientation not reliable; keeps flipping.
		//var imgOrientation = self._liveImg.naturalWidth > self._liveImg.naturalHeight ? "landscape" : "portrait";
		var screenOrientation = window.innerWidth > window.innerHeight ? "landscape" : "portrait";
		//g_genUI.TransientMsg("camera-orientation2-test", "s:" + screenOrientation + "; i:" + self._lastImgOrientation);
		if (self._lastImgOrientation != screenOrientation)
			g_genUI.TransientMsg("camera-orientation2", "Rotate your device (s:" + screenOrientation + "; i:" + self._lastImgOrientation + ")", true);
		else
			g_genUI.TransientMsg("camera-orientation2", "");
		
		if (window.orientation != self._prevOrientation) {
			self._prevOrientation = window.orientation;
			self.cameraRestart(function() {
				//g_genUI.TransientMsg("calling-avr", "calling-avr");
				self._afterVideoResize();
			});
		}
	}, 1000);
}

// Set maximum dimension of canvas according to _maxDim setting.
utilGenCam_svr.prototype._setCnvsDims = function(cnvs, origW, origH) {
	// Set XY dims
	var sourceHtoW = origH / origW;
	if (origH > origW) { // portrait
		cnvs.height = this._maxDim;
		cnvs.width = this._maxDim / sourceHtoW;
	}
	else { // landscape
		cnvs.width = this._maxDim;
		cnvs.height = this._maxDim * sourceHtoW;
	}
}

// TODO: rename _updateLiveCnvs()?
utilGenCam_svr.prototype._getImage = function(callback) {
	var self = this;

	if (!self._cameraSvrUrl) {
		g_genUI.TransientMsg("_getImage-err", "_getImage error: camera not initialized");
	}
	var image1 = new Image();
	image1.onload = function() {
		g_genUI.TransientMsg("_getImage-err2", ""); // Clear error (usually errors on startup :(.)
		//g_genUI.TransientMsg("natural w/h", image1.naturalWidth + ", " + image1.naturalHeight);
		var w = image1.naturalWidth;
		var h = image1.naturalHeight;
		self._lastImgOrientation = w > h ? "landscape" : "portrait";
		if (!self._gotFirstImage)
			self._gotFirstImage = true;
		if (self._liveCnvs) { // Update liveCanvas
			//self._liveCnvs.width = w;
			//self._liveCnvs.height = h;
			self._setCnvsDims(self._liveCnvs, w, h); // TODO: this could go in orientation-change instead, to do less work.
			self._liveCtx.drawImage(image1, 0, 0, self._liveCnvs.width, self._liveCnvs.height);
			//self._liveCtx.fillStyle="#0000FF";
			//self._liveCtx.fillRect(10,10,10,10);
		}
		callback();
	}
	image1.onerror = function() {
		g_genUI.TransientMsg("_getImage-err2", "Preview failed to load - restart?");
		// Try restarting shit. (This regularly comes into effect when screen wakes, iOS.)
		self.camServerRestart();
		self.startPreview();
	}
	image1.src = self._cameraSvrUrl + "live.jpg";
	//image1.src = "http://localhost:8080/live.jpg";
}

utilGenCam_svr.prototype._loopStream = function(onFrameLoad) {
	var self = this;
	self._getImage(function(imgUrl) {
		onFrameLoad(/*imgUrl*/);
		clearTimeout(self._loopTimer); // Clear existing loop, just in case.
		if (self._streaming) {
			self._loopTimer = setTimeout(function() { // Not really necessary.
				self._loopStream(onFrameLoad);
			}, 20);
		}
	});
}

utilGenCam_svr.prototype.startPreview = function() {
	//g_genUI.TransientMsg("svr-startPreview", "svr-startPreview 1");
	var self = this;
	self._streaming = true;
	//clearTimeout(self._loopTimer); // Clear existing loop, just in case.
	self._loopStream(function() {
		self._frameCount += 1;
		if (self.echoFrameCount)
			g_genUI.TransientMsg("svr-frames", "frames " + self._frameCount);
		//if (self._onFrameAdvance)
		//	self._onFrameAdvance();
	});
	//g_genUI.TransientMsg("svr-startPreview", "Camera-svr started");
}
utilGenCam_svr.prototype.stopPreview = function() {
	this._streaming = false;
	//clearTimeout(this._loopTimer);
	//g_genUI.TransientMsg("svr-startPreview", "svr-stopPreview");
}

utilGenCam_svr.prototype.frameCount = function() {
	return this._frameCount;
}

utilGenCam_svr.prototype.drawToCanvas = function(cnvs, ctx) {
	var self = this;
	
	if (!self._gotFirstImage) {
		g_genUI.TransientMsg("drawToCanvasSvr-stillstarting", "Camera still starting...");
		return;
	}
	else
		g_genUI.TransientMsg("drawToCanvasSvr-stillstarting", "");

	// Draw
	var w = self._liveCnvs.width;
	var h = self._liveCnvs.height;
	cnvs.width = w;
	cnvs.height = h;
	ctx.drawImage(self._liveCnvs, 0, 0, w, h);
	//ctx.fillStyle="#00FF00";
	//ctx.fillRect(0,0,10,10);
}

