"use strict";

// utilGenCamera
// Lot of glue holding RtcCamera (Android, browser) and SvrCamera (iOS) together, and abstracting the difference from the app code otherwise.
//--------------

function utilGenCamera(mode, $liveCnvs, opts) {
	var self = this;
	
	if (mode != "rtc" && mode != "svr") {
		alert("invalid genCamera mode");
	}
	this._mode = mode;
	this._$liveCnvs = $liveCnvs;
	if ($liveCnvs.length) {
		this._liveCnvs = $liveCnvs.get(0);
		this._liveCtx = this._liveCnvs.getContext('2d');
	}
	
	
	// Get optional intermediate elements
	this._afterVideoResize = opts.onVideoResize;
	//this._onFrameAdvance = opts.onFrameAdvance;
	this._rtcPreviewVideo = $(opts.$rtcPreviewVideo).get(0) || document.createElement('video');
	this._snapshotCnvs = opts.$snapshotCnvs.get(0) || document.createElement('canvas');
	this._maxDim = opts.maxDim || 800;
	
	this._rtcGumVideoVal = opts.rtcGumVideoVal || {optional: [
			{ facingMode: "environment" } // Default to rear cam - not supported by Android :(
		]};

	this._stgCnvs = this._snapshotCnvs; // snapshot staging canvas
	this._stgCtx = this._stgCnvs.getContext('2d');
	
	// TODO: require adapter.js somewhere!
	// TODO: get this from a better place.
	self.rtcCam = new utilGenCam_rtc(self._rtcGumVideoVal, null, self._rtcPreviewVideo, self._maxDim, self._afterVideoResize);

	this.svrCam = new utilGenCam_svr(this._$liveCnvs, this._maxDim, this._afterVideoResize);

	
}


utilGenCamera.prototype.startUp = function(callback) {
	var self = this;
	var genCam = this._mode == "svr" ? this.svrCam : this.rtcCam;

	genCam.startUp(function() {
		//self.startPreview(); // make consumer start the preview outside.
		if (callback)
			callback();
	});
}



utilGenCamera.prototype.startPreview = function() {
	var self = this;
	var genCam = this._mode == "svr" ? this.svrCam : this.rtcCam;
	genCam.startPreview();
	
	//if (this._onFrameAdvance && this._mode != "rtc") {
	//	alert("only rtc supports _onFrameAdvance");
	//}
	
	// Start painting preview canvas.
	// (only RTC.  svr-camera already outputs (more directly) to liveCnvs.)
	if (this._mode == "rtc" && self._liveCnvs) {
		var frameDelay = 50;
		this._previewPainter2 = function() {
			genCam.drawToCanvas(self._liveCnvs, self._liveCtx);
			//if (self._onFrameAdvance)
			//	self._onFrameAdvance();
			setTimeout(function() { self._previewPainter2() }, frameDelay);
		}
		this._previewPainter2();
	}

}
utilGenCamera.prototype.stopPreview = function() {
	if (this._mode == "svr")
		this.svrCam.stopPreview();
	else
		this.rtcCam.stopPreview();
	
	// Stop painting preview canvas.
	this._previewPainter2 = function() {}; // like clearTimeout().
}

utilGenCamera.prototype.frameCount = function() {
	var genCam = this._mode == "svr" ? this.svrCam : this.rtcCam;
	//g_genUI.TransientMsg("test", "test");
	return genCam.frameCount();
}

// Take a photo (get dataUrl).
utilGenCamera.prototype.takeDataUrl = function() {
	/*if (this._mode == "svr")
		this.svrCam.takeDataUrl();
	else
		this.rtcCam.takeDataUrl();*/
	
	var genCam = this._mode == "svr" ? this.svrCam : this.rtcCam;
	genCam.drawToCanvas(this._liveCnvs, this._liveCtx);
	//this._updateStagingCanvas();
	var dataUrl = this._liveCnvs.toDataURL('image/jpeg');
	//photo.setAttribute('src', data);
	return dataUrl;
}


// Take a photo (get output canvas).
utilGenCamera.prototype.takeBlob = function(callback) {
	
	// Homebrew function, until Chrome supports canvas.toBlob().
	// Asynchronous, like canvas.toBlob() will be.
	// (Taken from colorstream/aurora former chrome web app project.)
	var _toBlob = function(canvas, typeStr, callback) {
		var dataUrl = canvas.toDataURL( typeStr );
		var dataPart = dataUrl.substring(("data:" + typeStr + ";base64,").length );
		//console.log(dataPart.substr(0, 400));
		var data = atob( dataPart ),
			asArray = new Uint8Array(data.length);
		for( var i = 0, len = data.length; i < len; ++i ) {
			asArray[i] = data.charCodeAt(i);    
		}
		var blob = new Blob( [ asArray.buffer ], {type: typeStr} );
		callback(blob);
	}

	var genCam = this._mode == "svr" ? this.svrCam : this.rtcCam;
	genCam.drawToCanvas(this._stgCnvs, this._stgCtx);
	//this._updateStagingCanvas();
	_toBlob(this._stgCnvs, "image/jpeg", callback);
}


/*utilGenCam_rtc.prototype.toString = function () {

	var dataString = this.canvas.get(0).toDataURL("image/png");
	var index = dataString.indexOf( "," )+1;
	dataString = dataString.substring( index );
	
	return dataString;
}*/

/*utilGenCam_rtc.prototype.clear = function () {
	var c = this.canvas[0];
	this.context.clearRect( 0, 0, c.width, c.height );
}*/
